////////////////////////////////////////////////////////////////////////////////
// File         : RelationBaustein.h
// Subsystem    : Graph
// Component    : object networks
// Author       : Josef Huebl, (Josef.Huebl@sss.de)
// Description  : definition of the class RelationBaustein
////////////////////////////////////////////////////////////////////////////////
// Copyright (C) Triple-S GmbH (www.sss.de) 2003
// Alle Rechte vorbehalten. All Rights Reserved.
//
// The reproduction, transmission or use of this document or its contents is
// not permitted without express written authority.
// Offenders will be liable for damages. All rights, including rights created
// by patent grant or registration of a utility model or design, are reserved.
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// History:
// 10.10.03    Josef Huebl       created
// 17.12.03    Josef Huebl       1. released version.
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Hints:      
// 
// The class RelationBaustein is derived from Baustein and 
// manages the computing and storing relations.
// It is a base class for RelaisBaustein, where all modeling bausteins are derived from.  
//     
////////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_RELATIONBAUSTEIN_H__9730AAB1_D8A7_49D3_B594_058CF71FADCD__INCLUDED_)
#define AFX_RELATIONBAUSTEIN_H__9730AAB1_D8A7_49D3_B594_058CF71FADCD__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#ifdef WIN32
#pragma warning( disable : 4091 ) //warning C4091: 'extern ' : ignored on left of 'struct benes_t' when no variable is declared
#endif

#include "../../benessolver/benes.h"

#ifdef WIN32
#pragma warning( default : 4091 )  
#endif

typedef struct benes_t Benes_t;

#include "Baustein.h"

const int NO_ENTRY = -1;
const int NO_EXIT = -1;

class RelationBaustein : public Baustein  
{
public:
	RelationBaustein( bool injective, IsoTypeNode &class_isoType, int numberOfEntries, int numberOfExits, Baustein *owner, HnetUniverse *universe = NULL );
	virtual ~RelationBaustein();

public:
	virtual void ClearRelation( void ) = 0;
	virtual void ChangeRelation( int entryPos, int exitPos, int color ) = 0;

	void SetRelationsElementary( NodeAttribute< int > &color, const NodeAttribute< Relation > &relation );
	void SetMultibleFlagElementary( NodeAttribute< bool > &multible );
	void PrepareRelationComputation( void );
	bool DoRelationComputation( int defaultColor, int numberOfEntryColors, int *entryColors, int numberOfExitColors, int *exitColors );
	void FinishRelationComputation( void );

protected:
	void ComputePermutation( int *permutation );
	void ComputeBenesMatrix( int size, int *permutation, Benes_t **benes );
	void DeleteBenesMatrix( Benes_t *benes );

private:
	void SetRelation( NodeAttribute< int > &color, const Relation &relation );
	void SetMultibleFlags( NodeAttribute< bool > &multible, const Snet &snet );

private:
	bool m_injectiveRelation;  // true, if baustein can realise only injective relations

protected:
	std::vector< int > m_exitEntry;  // store a relation

};

#endif // !defined(AFX_RELATIONBAUSTEIN_H__9730AAB1_D8A7_49D3_B594_058CF71FADCD__INCLUDED_)
