////////////////////////////////////////////////////////////////////////////////
// File         : SnetPrint.h
// Subsystem    : Graph
// Component    : S-net
// Author       : Josef Huebl, (Josef.Huebl@sss.de)
// Description  : definition of the class SnetPrint
////////////////////////////////////////////////////////////////////////////////
// Copyright (C) Triple-S GmbH (www.sss.de) 2003
// Alle Rechte vorbehalten. All Rights Reserved.
//
// The reproduction, transmission or use of this document or its contents is
// not permitted without express written authority.
// Offenders will be liable for damages. All rights, including rights created
// by patent grant or registration of a utility model or design, are reserved.
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// History:
// 11.09.03    Josef Huebl       created
// 17.12.03    Josef Huebl       1. released version.
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Hints:      
//     
// This class adds unique numbers to the nodes the graph of a s-net and
// allows to print out a s-net with a single call of Print().
//     
////////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_SNETPRINT_H__A2CF77AA_49FF_462A_95B6_2D6842C0CA3D__INCLUDED_)
#define AFX_SNETPRINT_H__A2CF77AA_49FF_462A_95B6_2D6842C0CA3D__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "ObjPrint.h"
#include "NodesNumbers.h"
#include "GraphPrint.h"
#include "Snet.h"

#include <string.h>

class SnetPrint : public ObjPrint, public NodesNumbers, public Snet  
{
public:
	SnetPrint( const char *name = ObjPrintNoname );
	SnetPrint( SnetPrint &snet, const char *name = ObjPrintNoname);
	SnetPrint( Snet &snet, const char *name = ObjPrintNoname);
	virtual ~SnetPrint();
	SnetPrint& operator=( SnetPrint &snet);
	Node AddOuterEntry( void );
	Node AddOuterExit( void );
	Node AddSubcentre( int numEntries, int numExits );
	Node CopySubcentre( Node &s ) { return AddSubcentre( NumberOfEntries( s ), NumberOfExits( s ) ); };
	Node CopyAsSubcentre( Snet &snet ){ return AddSubcentre( snet.NumberOfOuterEntries(), snet.NumberOfOuterExits() ); };
	Node Replace( Node &s, SnetPrint &snet );
	void TransComplement( SnetPrint &complement, NodeAttribute< Node > &originNode ); 
	void Renumber( void ) { NodesNumbers::Renumber( Root() ); }
	void RenumberNew( void ) { NodesNumbers::RenumberNew( Root() ); }

	void PrintBody( void ) const;
	void PrintRelation( const NodeAttribute< Relation > &relation) const;
	void PrintNodePath( const NodePath &path ) { NodePathPrint p( *this, path ); p.Print(); }
	char *Str( const Node &n) const;

private:
	void PrintSubcenter( const Node &s ) const;
	void PrintConnections( const Node &s ) const;
	void Copy( SnetPrint &snet);

};

#endif // !defined(AFX_SNETPRINT_H__A2CF77AA_49FF_462A_95B6_2D6842C0CA3D__INCLUDED_)
