#include "search.h"
using namespace std;


void GraphSearch::start(const Graph &g, const Node &s)
{
	_g = &g;
	qe = QueueEl();
	queue.clear();
	if (s)
		queue.push_back(QueueEl(true, s));
	else
	{
		Node n;
		for (n=g.FirstNode(); n; n=n.NextNode())
			if (!myFirstIngoingEdge(n))	// erst alle Startknoten einfgen:
				queue.push_front(QueueEl(true, n));
			else	// dann Kreise beliebig aufbrechen:
				queue.push_back(QueueEl(true, n));
	}
	firstVisitingEdge.ReinitialiseValues();
	visited.ReinitialiseValues();
	Next();
}

void GraphSearch::Next(void)
{
	while (true)
	{
		if (queue.empty())
		{
			_g = 0;
			qe = QueueEl();
			break;	// finished
		}
		qe = queue.front();
		queue.pop_front();
		if (qe.entering)
		{
			qe.entering = false;
			if (!visited[qe.n])
			{
				visited[qe.n] = true;
				firstVisitingEdge[qe.n] = qe.e;
				if (dfs)
					queue.push_front(qe);
				Edge e;
				for (e=myFirstOutgoingEdge(qe.n); e; e=myNextOutgoingEdge(e))
					if (dfs)
						queue.push_front(QueueEl(true, myTargetNode(e), e));
					else // bfs
						queue.push_back(QueueEl(true, myTargetNode(e), e));
				if (!dfs)	// bfs
					queue.push_back(qe);
                		if (preHook)
                			break;	// break for user code
			}
		}
		else
		{
			if (!preHook)
               			break;	// break for user code
		}
	}
	n = qe.n;
	e = qe.e;
}

bool GraphSearch::done(void)
{
	return _g == 0;
}

Node GraphSearch::getCurrentNode(void)
{
	return qe.n;
}

Edge GraphSearch::getCurrentEdge(void)
{
	return qe.e;
}

Edge GraphSearch::getParentEdge(const Node &n)
{
	return firstVisitingEdge[n];
}

// wrappers follow...

void GraphSearch::startDFS(const Graph &g, const Node &s)
{
	fwd = true;
	dfs = true;
	preHook = true;
	start(g, s);
}

void GraphSearch::startDFSPost(const Graph &g, const Node &s)
{
	fwd = true;
	dfs = true;
	preHook = false;
	start(g, s);
}

void GraphSearch::startBFS(const Graph &g, const Node &s)
{
	fwd = true;
	dfs = false;
	preHook = true;
	start(g, s);
}

void GraphSearch::startBFSPost(const Graph &g, const Node &s)
{
	fwd = true;
	dfs = false;
	preHook = false;
	start(g, s);
}

void GraphSearch::startBwdDFS(const Graph &g, const Node &t)
{
	fwd = false;
	dfs = true;
	preHook = true;
	start(g, t);
}

void GraphSearch::startBwdDFSPost(const Graph &g, const Node &t)
{
	fwd = false;
	dfs = true;
	preHook = false;
	start(g, t);
}

void GraphSearch::startBwdBFS(const Graph &g, const Node &t)
{
	fwd = false;
	dfs = false;
	preHook = true;
	start(g, t);
}

void GraphSearch::startBwdBFSPost(const Graph &g, const Node &t)
{
	fwd = false;
	dfs = false;
	preHook = false;
	start(g, t);
}

// silly internal functions...

Node GraphSearch::myTargetNode(Edge &e)
{
	if (fwd)
		return e.TargetNode();
	else
		return e.SourceNode();
}

Edge GraphSearch::myFirstIngoingEdge(Node &n)
{
	if (fwd)
		return n.FirstIngoingEdge();
	else
		return n.FirstOutgoingEdge();
}

Edge GraphSearch::myFirstOutgoingEdge(Node &n)
{
	if (fwd)
		return n.FirstOutgoingEdge();
	else
		return n.FirstIngoingEdge();
}

Edge GraphSearch::myNextOutgoingEdge(Edge &e)
{
	if (fwd)
		return e.NextOutgoingEdge();
	else
		return e.NextIngoingEdge();
}
